function protocol=MMConstants(model,protocol)
% protocol = MMConstants(model,protocol)
%
% adds to protocol all the constants and parameters necessary for
% calculating the diffusion signal usin Matrix Method
%
% input: 
% model - structure describing the diffusion substrate
% protocol - structure describing the diffusion imaging parameters
%
% author: Andrada Ianus (a.ianus.11@ucl.ac.uk), Ivana Drobnjak (i.drobnjak@ucl.ac.uk), Daniel
% C. Alexander (d.alexander@ucl.ac.uk)
%

% Setting qunit, N, tau, gstep
GAMMA = 2.675987E8;

if ~isfield(protocol,'dimA')
  protocol.dimA=20;
end
if ~isfield(protocol,'diff')
  protocol.diff=1; % calculates the derivatives of the matrices
end
if ~isfield(protocol,'tau')
    if strcmp(protocol.pulseseq,'GEN') % needed when optimising each point on the waveform;
        protocol.tau=protocol.riset*(2+0.1);
    else
        protocol.tau=1E-4;
    end
end
if ~isfield(protocol,'mirror')
  protocol.mirror=0; % when 1 the gradient waveform is mirrored about the 180 degree pulse
end
if ~isfield(protocol,'complex')
    protocol.complex='abs'; % 
end
if isfield(protocol,'smalldel') && isfield(protocol,'delta') 
protocol.K = floor((protocol.smalldel+1E-10)./protocol.tau)+1;
protocol.dstmp = floor((protocol.delta-1E-10)./protocol.tau)-floor((protocol.smalldel+1E-10)./protocol.tau);
end
protocol.gstep=0.04/100;
protocol.qunit=GAMMA*protocol.tau*protocol.gstep;
dimA = protocol.dimA;
qunit=protocol.qunit/(2*pi);
protocol.pert=0.00001;

strings = GetParameterStrings(model.name);

if sum(strcmp('rad',strings))  % cylinder restriction
    radind = GetParameterIndex(model.name,'rad');
    [urad_vec urad_ind] = unique(model.params(:,radind), 'stable');
    modelcode = 1; % cylinder
    for j = 1:length(urad_vec)
        D = model.params(urad_ind(j),GetParameterIndex(model.name,'di')); % diffusivity
        a = model.params(urad_ind(j),GetParameterIndex(model.name,'rad')); % rad
        roo=MMroots(dimA,a,modelcode); % cylinder
        [protocol.S_cyl{j} protocol.A_cyl{j} protocol.kDn_cyl{j}]=MatrixSA(qunit,a,dimA,roo,modelcode);
        protocol.R_cyl{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
        if protocol.diff==1
            % Radius apert
            apert=a*(1+protocol.pert);
            roo=MMroots(dimA,apert,modelcode);
            [protocol.Sperta_cyl{j} protocol.Aperta_cyl{j} protocol.kDn_cyl{j}]=MatrixSA(qunit,apert,dimA,roo,modelcode);
            protocol.Rperta_cyl{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D); 
            % Dpert
            Dpert=D*(1+protocol.pert);
            roo=MMroots(dimA,a,modelcode);
            protocol.RpertD_cyl{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
        end
    end
end
if sum(strcmp('lcyl',strings))  % planar restriction fot finite cylinders
    modelcode = 2; % planar
    for j = 1:length(urad_vec)
        D = model.params(urad_ind(j),GetParameterIndex(model.name,'di')); % diffusivity
        l = model.params(urad_ind(j),GetParameterIndex(model.name,'lcyl'))/2; % half the length; in the code the parallel planes are situated between -l and l
        roo=MMroots(dimA,l,modelcode); % planar
        [protocol.S_plane{j} protocol.A_plane{j} ]=MatrixSA(qunit,l,dimA,roo,modelcode);
        protocol.R_plane{j}=MatrixR(protocol.tau,l,dimA,roo,modelcode,D);
        if protocol.diff==1
            % Dpert
            Dpert = D*(1+protocol.pert);
            roo=MMroots(dimA,l,modelcode);
            protocol.RpertD_plane{j}=MatrixR(protocol.tau,l,dimA,roo,modelcode,Dpert); 
            % L perp
            lpert=l*(1+protocol.pert);
            roo=MMroots(dimA,lpert,modelcode);
            [protocol.Spertl_plane{j} protocol.Apertl_plane{j}]=MatrixSA(qunit,lpert,dimA,roo,modelcode);
            protocol.Rpertl_plane{j}=MatrixR(protocol.tau,lpert,dimA,roo,modelcode,D); 
        
        end
    end
end
if sum(strcmp('rads',strings))  % spherical restriction
    radind = GetParameterIndex(model.name,'rads');
    [urad_vec urad_ind] = unique(model.params(:,radind), 'stable');
    modelcode = 0; % cylinder
    for j = 1:length(urad_vec)
        D = model.params(urad_ind(j),GetParameterIndex(model.name,'di')); % diffusivity
        a = model.params(urad_ind(j),GetParameterIndex(model.name,'rads')); % rad
        roo=MMroots(dimA,a,modelcode); % cylinder
        [Stemp Atemp protocol.kDn_sph]=MatrixSA(qunit,a,dimA,roo,modelcode);
        protocol.A_sph{j,1} = Atemp{1}; % A0
        protocol.A_sph{j,2} = Atemp{2}; % A90
        protocol.S_sph{j,1} = Stemp{1}; % S0
        protocol.S_sph{j,2} = Stemp{2}; % S90
        protocol.R_sph{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
        if protocol.diff==1
            % Radius apert
            apert=a*(1+protocol.pert);
            roo=MMroots(dimA,apert,modelcode);
            [Stemp Atemp ]=MatrixSA(qunit,apert,dimA,roo,modelcode);
            protocol.Aperta_sph{j,1} = Atemp{1}; % A0
            protocol.Aperta_sph{j,2} = Atemp{2}; % A90
            protocol.Sperta_sph{j,1} = Stemp{1}; % S0
            protocol.Sperta_sph{j,2} = Stemp{2}; % S90
            protocol.Rperta_sph{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D); 
            % Dpert
            Dpert=D*(1+protocol.pert);
            roo=MMroots(dimA,a,modelcode);
            protocol.RpertD_sph{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
        end
    end
end  
if (sum(strcmp('lx',strings))+sum(strcmp('ly',strings))+sum(strcmp('lz',strings)))==3  % cuboids
    lxind = GetParameterIndex(model.name,'lx');
    lyind = GetParameterIndex(model.name,'ly');
    lzind = GetParameterIndex(model.name,'lz');
    [ul_vec ul_ind] = unique([model.params(:,lxind) model.params(:,lyind) model.params(:,lzind)],'rows','stable');
    modelcode = 2; % planar
    for j = 1:length(ul_ind)        
        D = model.params(ul_ind(j),GetParameterIndex(model.name,'di')); % diffusivity
        a = model.params(ul_ind(j),GetParameterIndex(model.name,'lx'))/2;  % lx/2
        b = model.params(ul_ind(j),GetParameterIndex(model.name,'ly'))/2;  % ly/2
        c = model.params(ul_ind(j),GetParameterIndex(model.name,'lz'))/2;  % lz/2
        roo=MMroots(dimA,a,modelcode); % x dir
        [protocol.S_x{j} protocol.A_x{j} ]=MatrixSA(qunit,a,dimA,roo,modelcode);
        roo = MMroots(dimA,b,modelcode); % y dir
        [protocol.S_y{j} protocol.A_y{j} ]=MatrixSA(qunit,b,dimA,roo,modelcode);
        roo = MMroots(dimA,c,modelcode); % z dir
        [protocol.S_z{j} protocol.A_z{j} ]=MatrixSA(qunit,c,dimA,roo,modelcode);   
        % from roo MatrixR takes only the eigenvalues which are the same
        % no mater what length
        protocol.R_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
        protocol.R_y{j}=MatrixR(protocol.tau,b,dimA,roo,modelcode,D);
        protocol.R_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,D);
        if protocol.diff==1
            % lx pert
            apert=a*(1+protocol.pert);
            roo=MMroots(dimA,apert,modelcode);
            [protocol.Sperta_x{j} protocol.Aperta_x{j} ]=MatrixSA(qunit,apert,dimA,roo,modelcode);
            % ly pert
            bpert=b*(1+protocol.pert);
            roo=MMroots(dimA,bpert,modelcode);
            [protocol.Spertb_y{j} protocol.Apertb_y{j} ]=MatrixSA(qunit,bpert,dimA,roo,modelcode);
            %lz pert
            cpert=c*(1+protocol.pert);
            roo=MMroots(dimA,cpert,modelcode);
            [protocol.Spertc_z{j} protocol.Apertc_z{j} ]=MatrixSA(qunit,cpert,dimA,roo,modelcode);
      
            protocol.Rperta_x{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D); 
            protocol.Rpertb_y{j}=MatrixR(protocol.tau,bpert,dimA,roo,modelcode,D); 
            protocol.Rpertc_z{j}=MatrixR(protocol.tau,cpert,dimA,roo,modelcode,D); 
            % Dpert
            Dpert=D*(1+protocol.pert);
            protocol.RpertD_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
            protocol.RpertD_y{j}=MatrixR(protocol.tau,b,dimA,roo,modelcode,Dpert); 
            protocol.RpertD_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,Dpert); 
        end
    end
end
if (sum(strcmp('lx',strings))+sum(strcmp('ecc',strings))) == 2  % cuboids with lx = ly
    lxind = GetParameterIndex(model.name,'lx');
    eccind = GetParameterIndex(model.name,'ecc');   
    [ul_vec ul_ind] = unique([model.params(:,lxind) model.params(:,eccind)],'rows','stable');
    modelcode = 2; % planar
    for j = 1:length(ul_ind)        
        D = model.params(ul_ind(j),GetParameterIndex(model.name,'di')); % diffusivity
        a = model.params(ul_ind(j),GetParameterIndex(model.name,'lx'))/2;  % lx/2
        Ecc = model.params(ul_ind(j),GetParameterIndex(model.name,'ecc'));  % ecc        
        c = Ecc*a;
        roo=MMroots(dimA,a,modelcode); % x dir
        [protocol.S_x{j} protocol.A_x{j} ]=MatrixSA(qunit,a,dimA,roo,modelcode);       
        protocol.S_y{j} = protocol.S_x{j};
        protocol.A_y{j} = protocol.A_x{j};
        
        roo = MMroots(dimA,c,modelcode); % x dir
        [protocol.S_z{j} protocol.A_z{j} ]=MatrixSA(qunit,c,dimA,roo,modelcode);   
        % from roo MatrixR takes only the eigenvalues which are the same
        % no mater what length
        protocol.R_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
        protocol.R_y{j}=protocol.R_x{j};
        protocol.R_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,D);
        if protocol.diff==1
            % lx = ly pert
            apert=a*(1+protocol.pert);
            roo=MMroots(dimA,apert,modelcode);
            [protocol.Sperta_x{j} protocol.Aperta_x{j} ]=MatrixSA(qunit,apert,dimA,roo,modelcode);
            protocol.Rperta_x{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D);  
            protocol.Sperta_y{j} = protocol.Sperta_x{j};
            protocol.Aperta_y{j} = protocol.Aperta_x{j};
            protocol.Rperta_y{j} = protocol.Rperta_x{j};
             cpert = apert*Ecc;
             roo=MMroots(dimA,cpert,modelcode);
             [protocol.Sperta_z{j} protocol.Aperta_z{j} ]=MatrixSA(qunit,cpert,dimA,roo,modelcode);
             protocol.Rperta_z{j}=MatrixR(protocol.tau,cpert,dimA,roo,modelcode,D);  
          
            %Ecc pert
            Eccpert = Ecc*(1+protocol.pert);
            cpert=Eccpert*a;
            roo=MMroots(dimA,cpert,modelcode);
            [protocol.SpertEcc_z{j} protocol.ApertEcc_z{j} ]=MatrixSA(qunit,cpert,dimA,roo,modelcode);
            protocol.RpertEcc_z{j}=MatrixR(protocol.tau,cpert,dimA,roo,modelcode,D); 
         
            % Dpert
            Dpert=D*(1+protocol.pert);
            protocol.RpertD_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
            protocol.RpertD_y{j}=protocol.RpertD_x{j}; 
            protocol.RpertD_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,Dpert); 
        end
    end
end
 protocol.diff=0; %Used in Synth
 end



% numCombs = size(model.restriction_params,1);
% % Calculating the matrices
% numres = length(model.restriction_name); % model.restriction is a cell array of restricted compartments
% for j = 1:numCombs
%      params=model.restriction_params(j,:); % parameters associated to restriction;
%      %can be different to the total model parameters when there is a composite model
%     for nr = 1:numres
%     if strcmp(model.restriction_name{nr},'Cylinder') % two parameters - diffusivity and radius
%         modelcode = 1; % cylinder
%         D = params(1); % diffusivity
%         a = params(2); % radius
%         roo=MMroots(dimA,a,modelcode); % cylinder
%         [protocol.S_cyl{j} protocol.A_cyl{j} protocol.kDn_cyl{j}]=MatrixSA(qunit,a,dimA,roo,modelcode);
%         protocol.R_cyl{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
%         if protocol.diff==1
%             % Radius apert
%             apert=a*(1+protocol.pert);
%             roo=MMroots(dimA,apert,modelcode);
%             [protocol.Sperta_cyl{j} protocol.Aperta_cyl{j} protocol.kDn_cyl{j}]=MatrixSA(qunit,apert,dimA,roo,modelcode);
%             protocol.Rperta_cyl{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D); 
%             % Dpert
%             Dpert=D*(1+protocol.pert);
%             roo=MMroots(dimA,a,modelcode);
%             protocol.RpertD_cyl{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
%         end
%         
%     elseif strcmp(model.restriction_name{nr},'FiniteCylinder')
%         D = params(1); % diffusivity
%         a = params(2); % radius
%         l = params(3)/2; % half cylinder length; in the code the parallel planes are situated between -l and l
%         roo=MMroots(dimA,a,1); % cylinder
%         [protocol.S_cyl{j} protocol.A_cyl{j} protocol.kDn_cyl{j}]=MatrixSA(qunit,a,dimA,roo,1);
%         protocol.R_cyl{j}=MatrixR(protocol.tau,a,dimA,roo,1,D);
%         
%         roo=MMroots(dimA,l,2); % parallel planes
%         [protocol.S_plane{j} protocol.A_plane{j} ]=MatrixSA(qunit,l,dimA,roo,2);
%         protocol.R_plane{j}=MatrixR(protocol.tau,l,dimA,roo,2,D);
%          if protocol.diff==1
%             % Radius apert
%             apert=a*(1+protocol.pert);
%             roo=MMroots(dimA,apert,1);
%             [protocol.Sperta_cyl{j} protocol.Aperta_cyl{j} protocol.kDn_cyl{j}]=MatrixSA(qunit,apert,dimA,roo,1);
%             protocol.Rperta_cyl{j}=MatrixR(protocol.tau,apert,dimA,roo,1,D); 
%             % Dpert
%             % cylinder
%             Dpert=D*(1+protocol.pert);
%             protocol.RpertD_cyl{j}=MatrixR(protocol.tau,a,dimA,roo,1,Dpert); 
%             % plane
%             roo=MMroots(dimA,l,2);
%             protocol.RpertD_plane{j}=MatrixR(protocol.tau,l,dimA,roo,2,Dpert); 
%             % L perp
%             lpert=l*(1+protocol.pert);
%              roo=MMroots(dimA,lpert,2);
%             [protocol.Spertl_plane{j} protocol.Apertl_plane{j}]=MatrixSA(qunit,lpert,dimA,roo,2);
%             protocol.Rpertl_plane{j}=MatrixR(protocol.tau,lpert,dimA,roo,2,D); 
%          end        
%         
%     elseif strcmp(model.restriction_name{nr},'Sphere')
%         modelcode = 0; % sphere
%         D = params(1); % diffusivity
%         a = params(2); % radius
%         roo=MMroots(dimA,a,modelcode); % sphere
%         [Stemp Atemp protocol.kDn_sph]=MatrixSA(qunit,a,dimA,roo,modelcode);
%         protocol.A_sph{j,1} = Atemp{1}; % A0
%         protocol.A_sph{j,2} = Atemp{2}; % A90
%         protocol.S_sph{j,1} = Stemp{1}; % S0
%         protocol.S_sph{j,2} = Stemp{2}; % S90
%         protocol.R_sph{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
%         if protocol.diff==1
%             % Radius apert
%             apert=a*(1+protocol.pert);
%             roo=MMroots(dimA,apert,modelcode);
%             [Stemp Atemp ]=MatrixSA(qunit,apert,dimA,roo,modelcode);
%             protocol.Aperta_sph{j,1} = Atemp{1}; % A0
%             protocol.Aperta_sph{j,2} = Atemp{2}; % A90
%             protocol.Sperta_sph{j,1} = Stemp{1}; % S0
%             protocol.Sperta_sph{j,2} = Stemp{2}; % S90
%             protocol.Rperta_sph{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D); 
%             % Dpert
%             Dpert=D*(1+protocol.pert);
%             roo=MMroots(dimA,a,modelcode);
%             protocol.RpertD_sph{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
%         end
%     elseif strcmp(model.restriction_name{nr},'Cuboid')
%         modelcode = 2; % parallel planes E = Ex*Ey*Ez
%         D = params(1); % diffusivity
%         a = params(2)/2; % lx/2
%         b = params(3)/2; % ly/2
%         c = params(4)/2; % lz/2
%         roo=MMroots(dimA,a,modelcode); % x dir
%         [protocol.S_x{j} protocol.A_x{j} ]=MatrixSA(qunit,a,dimA,roo,modelcode);
%         roo = MMroots(dimA,b,modelcode); % x dir
%         [protocol.S_y{j} protocol.A_y{j} ]=MatrixSA(qunit,b,dimA,roo,modelcode);
%         roo = MMroots(dimA,c,modelcode); % x dir
%         [protocol.S_z{j} protocol.A_z{j} ]=MatrixSA(qunit,c,dimA,roo,modelcode);   
%         % from roo MatrixR takes only the eigenvalues which are the same
%         % no mater what length
%         protocol.R_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
%         protocol.R_y{j}=MatrixR(protocol.tau,b,dimA,roo,modelcode,D);
%         protocol.R_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,D);
%         if protocol.diff==1
%             % lx pert
%             apert=a*(1+protocol.pert);
%             roo=MMroots(dimA,apert,modelcode);
%             [protocol.Sperta_x{j} protocol.Aperta_x{j} ]=MatrixSA(qunit,apert,dimA,roo,modelcode);
%             % ly pert
%             bpert=b*(1+protocol.pert);
%             roo=MMroots(dimA,bpert,modelcode);
%             [protocol.Spertb_y{j} protocol.Apertb_y{j} ]=MatrixSA(qunit,bpert,dimA,roo,modelcode);
%             %lz pert
%             cpert=c*(1+protocol.pert);
%             roo=MMroots(dimA,cpert,modelcode);
%             [protocol.Spertc_z{j} protocol.Apertc_z{j} ]=MatrixSA(qunit,cpert,dimA,roo,modelcode);
%       
%             protocol.Rperta_x{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D); 
%             protocol.Rpertb_y{j}=MatrixR(protocol.tau,bpert,dimA,roo,modelcode,D); 
%             protocol.Rpertc_z{j}=MatrixR(protocol.tau,cpert,dimA,roo,modelcode,D); 
%             % Dpert
%             Dpert=D*(1+protocol.pert);
%             protocol.RpertD_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
%             protocol.RpertD_y{j}=MatrixR(protocol.tau,b,dimA,roo,modelcode,Dpert); 
%             protocol.RpertD_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,Dpert); 
%         end
%         
%     elseif strcmp(model.restriction_name{nr},'CuboidSym')  
%         modelcode = 2; % parallel planes E = Ex*Ey*Ez
%         D = params(1); % diffusivity
%         a = params(2)/2; % lx/2
%         Ecc = params(3);  % the last parameter is Ecc = lz/lx
%         c = Ecc*a;
%         roo=MMroots(dimA,a,modelcode); % x dir
%         [protocol.S_x{j} protocol.A_x{j} ]=MatrixSA(qunit,a,dimA,roo,modelcode);       
%         protocol.S_y{j} = protocol.S_x{j};
%         protocol.A_y{j} = protocol.A_x{j};
%         
%         roo = MMroots(dimA,c,modelcode); % x dir
%         [protocol.S_z{j} protocol.A_z{j} ]=MatrixSA(qunit,c,dimA,roo,modelcode);   
%         % from roo MatrixR takes only the eigenvalues which are the same
%         % no mater what length
%         protocol.R_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,D);
%         protocol.R_y{j}=protocol.R_x{j};
%         protocol.R_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,D);
%         if protocol.diff==1
%             % lx = ly pert
%             apert=a*(1+protocol.pert);
%             roo=MMroots(dimA,apert,modelcode);
%             [protocol.Sperta_x{j} protocol.Aperta_x{j} ]=MatrixSA(qunit,apert,dimA,roo,modelcode);
%             protocol.Rperta_x{j}=MatrixR(protocol.tau,apert,dimA,roo,modelcode,D);  
%             protocol.Sperta_y{j} = protocol.Sperta_x{j};
%             protocol.Aperta_y{j} = protocol.Aperta_x{j};
%             protocol.Rperta_y{j} = protocol.Rperta_x{j};
%              cpert = apert*Ecc;
%              roo=MMroots(dimA,cpert,modelcode);
%              [protocol.Sperta_z{j} protocol.Aperta_z{j} ]=MatrixSA(qunit,cpert,dimA,roo,modelcode);
%              protocol.Rperta_z{j}=MatrixR(protocol.tau,cpert,dimA,roo,modelcode,D);  
%           
%             %Ecc pert
%             Eccpert = Ecc*(1+protocol.pert);
%             cpert=Eccpert*a;
%             roo=MMroots(dimA,cpert,modelcode);
%             [protocol.SpertEcc_z{j} protocol.ApertEcc_z{j} ]=MatrixSA(qunit,cpert,dimA,roo,modelcode);
%             protocol.RpertEcc_z{j}=MatrixR(protocol.tau,cpert,dimA,roo,modelcode,D); 
%          
%             % Dpert
%             Dpert=D*(1+protocol.pert);
%             protocol.RpertD_x{j}=MatrixR(protocol.tau,a,dimA,roo,modelcode,Dpert); 
%             protocol.RpertD_y{j}=protocol.RpertD_x{j}; 
%             protocol.RpertD_z{j}=MatrixR(protocol.tau,c,dimA,roo,modelcode,Dpert); 
%         end
%     else
%         error('Unknown restriction for Matrix Method')
%     end
%     end
% end    
% protocol.diff=0; %Used in Synth
% end
