function [wave_form]= wave_form(protocol)
% wave_form = wave_form(protocol)
% Generates discrete gradient waveforms according to the diffusion protocol
% that can be used as input to synthesize the diffusion signal using 
% Matrix Method 
% suported pulse sequences in protocol: PGSE, OGSE, SWOGSE, TWOGSE, SWOGSE_3D
% dPFG, STEAM, Helical
%
% authors: Andrada Ianus (a.ianus.11@ucl.ac.uk)
% Ivana Drobnjak (i.drobnjak@ucl.ac.uk)
%
% $Id$
if ~isfield(protocol,'tau')
    protocol.tau = 1E-4;
end
if strcmp(protocol.pulseseq, 'PGSE') || strcmp(protocol.pulseseq, 'OGSE') || strcmp(protocol.pulseseq, 'SWOGSE') || ...
        strcmp(protocol.pulseseq, 'TWOGSE') || strcmp(protocol.pulseseq, 'SWOGSE_3D') % sequences based on spin echo sequences
    if isfield(protocol,'totalmeas')
        M = protocol.totalmeas;
    else
        M = length(protocol.smalldel);
    end
    total_time = max(ceil((protocol.smalldel+protocol.delta)/protocol.tau)).*protocol.tau+protocol.tau;
    time_vec = 0:protocol.tau:total_time;
    wav = zeros(3,length(time_vec));
    wave_form=zeros(M,length(wav(:)));
end

if(strcmp(protocol.pulseseq, 'PGSE'))
    for j = 1:M
        dG =protocol.G(j).*protocol.grad_dirs(j,:);
        for i=2:length(time_vec)
            if(time_vec(i)-protocol.smalldel(j)<-1E-10)
               wav(:,i) = dG;
               i_smalldel = i+1;
               i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
            elseif(time_vec(i)-protocol.delta(j)<-1E-10)
               wav(:,i) = 0;
               i_delta = i;
            elseif i-i_delta <=i_smalldel % operate on integers
                if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests                    
                    wav(:,i) = -wav(:,i-i_delta+1);
                else                    
                    wav(:,i) = -wav(:,i-i_delta);                    
                end
            else
                wav(:,i) = 0;
            end
        end
         wave_form(j,:) = wav(:);
    end
    

elseif(strcmp(protocol.pulseseq, 'SWOGSE')) 
     if(isfield(protocol,'omega'))
        for j = 1:M
        dG =protocol.G(j).*protocol.grad_dirs(j,:);
         
            if ~isfield(protocol,'phase')  % no phase            
                for i=2:length(time_vec)
                    if(time_vec(i)-protocol.smalldel(j)<-1E-10)       
                        wav(:,i) =dG*(-1).^floor(protocol.omega(j)./pi()*time_vec(i)-1E-10);
                        i_smalldel = i+1;
                        i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
                    elseif(time_vec(i)-protocol.delta(j)<-1E-10)    
                        wav(:,i) = 0;
                        i_delta = i;
                   elseif i-i_delta <=i_smalldel % operate on integers
                       if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                             if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta+1);
                             else
                                 wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                             end
                        else
                             if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta);
                             else
                                wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                             end
                        end
                    else
                        wav(:,i) = 0;
                    end  
                end            
            else           
                for i=2:length(time_vec)                     
                    if(time_vec(i)-protocol.smalldel(j)<-1E-10)
                          wav(:,i) =  dG*(-1).^floor((protocol.omega(j)*time_vec(i)-protocol.phase(j))./pi()-1E-10);
                          i_smalldel = i+1;
                          i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
                    elseif(time_vec(i)-protocol.delta(j)<-1E-10)
                          wav(:,i) = 0;
                          i_delta =  i;
                    elseif i-i_delta <=i_smalldel % operate on integers
                        if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                             if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta+1);
                             else
                                 wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                             end
                        else
                             if  ~isfield(protocol,'mirror') || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta);
                             else
                                wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                             end
                        end
                    else
                         wav(:,i) = 0;
                    end
                end           
            end
         wave_form(j,:) = wav(:);
        end
      else
        error('the protocol does not contain enough information to generate SWOGSE');
     end
elseif(strcmp(protocol.pulseseq, 'OGSE')) 
    if ~isfield(protocol,'apodisedcos') || protocol.apodisedcos == 0
     if(isfield(protocol,'omega'))
        for j = 1:M
        dG =protocol.G(j).*protocol.grad_dirs(j,:);
         
            if ~isfield(protocol,'phase')  % no phase            
                for i=2:length(time_vec)
                    if(time_vec(i)-protocol.smalldel(j)<-1E-10)       
                        wav(:,i) =dG*sin(protocol.omega(j).*time_vec(i));
                        i_smalldel = i+1;
                        i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
                    elseif(time_vec(i)-protocol.delta(j)<-1E-10)    
                        wav(:,i) = 0;
                        i_delta = i;
                   elseif i-i_delta <=i_smalldel % operate on integers
                       if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                             if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta+1);
                             else
                                 wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                             end
                        else
                             if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta);
                             else
                                wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                             end
                        end
                    else
                        wav(:,i) = 0;
                    end  
                end            
            else           
                for i=2:length(time_vec)                     
                    if(time_vec(i)-protocol.smalldel(j)<-1E-10)
                          wav(:,i) =  dG*sinr(protocol.omega(j)*time_vec(i)-protocol.phase(j));
                          i_smalldel = i+1;
                          i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
                    elseif(time_vec(i)-protocol.delta(j)<-1E-10)
                          wav(:,i) = 0;
                          i_delta =  i;
                    elseif i-i_delta <=i_smalldel % operate on integers
                        if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                             if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta+1);
                             else
                                 wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                             end
                        else
                             if  ~isfield(protocol,'mirror') || protocol.mirror ==0 % repeated
                                wav(:,i) = -wav(:,i-i_delta);
                             else
                                wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                             end
                        end
                    else
                         wav(:,i) = 0;
                    end
                end           
            end
         wave_form(j,:) = wav(:);
        end
      else
        error('the protocol does not contain enough information to generate SWOGSE');
     end 
    else % returns the waveforms for apodised cosine (Does MRM 2003)
        
        for j = 1:M
        dG =protocol.G(j).*protocol.grad_dirs(j,:);
        N = floor(protocol.omega(j).*protocol.smalldel(j)/pi+0.0000000001); 
        if protocol.omega(j).*protocol.smalldel(j)/pi - floor(protocol.omega(j).*protocol.smalldel(j)/pi+0.0000000001) > 1E-4; 
            error('apodised cosine works only for waveforms with integer number of lobebs. To use this set the appropriate omega.')
        end
        if isfield(protocol,'phase')
            if ~(abs(protocol.phase(j) -pi/2) < 1E-4 || abs(protocol.phase(j) + pi/2) < 1E-4) 
                error('apodisation works only for cosine waveforms, i.e. phase = pi/2 or -pi/2; for a different phase protocol.apodisedcos must be 0')
            end
        end
        for i=1:length(time_vec)

            if(time_vec(i)-pi/protocol.omega(j)/2<-1E-10)
               wav(:,i) = dG*sin(2*protocol.omega(j)*time_vec(i));
            elseif(time_vec(i)-(protocol.smalldel(j)-pi/protocol.omega(j)/2)<-1E-10)
               wav(:,i) = -dG*sin(protocol.omega(j)*time_vec(i)-pi/2);
            elseif(time_vec(i)-protocol.smalldel(j)<-1E-10)
                wav(:,i) = (-1).^N.*dG*sin(2*protocol.omega(j)*(time_vec(i)-protocol.smalldel(j))+pi);
                i_smalldel = i+1;
                i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
            elseif(time_vec(i)-protocol.delta(j)<-1E-10)
                wav(:,i) = 0;
                 i_delta =  i;
           elseif i-i_delta <=i_smalldel % operate on integers
                if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                     if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                        wav(:,i) = -wav(:,i-i_delta+1);
                     else
                         wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                     end
                else
                     if  ~isfield(protocol,'mirror') || protocol.mirror ==0 % repeated
                        wav(:,i) = -wav(:,i-i_delta);
                     else
                        wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                     end
                end
            else
                 wav(:,i) = 0;
            end
        end
         wave_form(j,:) = wav(:);
        end        
    end
     
elseif strcmp(protocol.pulseseq,'TWOGSE')  % does not have phase yet
    if ~isfield(protocol,'apodisedcos') || protocol.apodisedcos == 0
    if(isfield(protocol,'slew_rate') && isfield(protocol,'omega') )
        rt_vec = protocol.G./protocol.slew_rate; % rise time  
        for j = 1:M  
            if ~isfield(protocol,'phase')  % no phase        
                rt = rt_vec(j);
                Nt = floor(protocol.smalldel(j).*protocol.omega(j)./pi+0.00000000001);
                dG =protocol.G(j).*protocol.grad_dirs(j,:);
                
                    for i=2:length(time_vec)            
                        if(time_vec(i)<Nt.*pi/protocol.omega(j))
                            it = floor(protocol.omega(j)*time_vec(i)./pi()+0.00000001);
                            if( time_vec(i)<it *pi/protocol.omega(j)+rt)
                                wav(:,i) = dG./rt.*(-1).^it.*(time_vec(i)-it.*pi./protocol.omega(j));

                            elseif( time_vec(i)<(it+1)*pi/protocol.omega(j)-rt)
                                wav(:,i) = dG*(-1)^it;
                            else                   
                                wav(:,i) = dG./rt.*(-1).^it.*((it+1).*pi./protocol.omega(j)-time_vec(i));
                            end
                        elseif(time_vec(i)<protocol.smalldel(j)) 
                            if (protocol.smalldel(j)-Nt.*pi/protocol.omega(j)>2*rt)  % last oscillation longer than 2 *rt
                                if( time_vec(i)<Nt.*pi/protocol.omega(j)+rt)
                                    wav(:,i) = dG./rt.*(-1).^Nt.*(time_vec(i)-Nt.*pi./protocol.omega(j));

                                elseif( time_vec(i)<protocol.smalldel(j)-rt)
                                    wav(:,i) = dG*(-1)^Nt;
                                else
                                    wav(:,i) = dG./rt.*(-1).^Nt.*(protocol.smalldel(j)-time_vec(i));
                                end
                            else  % last oscillation shorter than 2 *rt
                                if( time_vec(i)<Nt.*pi/protocol.omega(j)+(protocol.smalldel(j)-Nt.*pi/protocol.omega(j))/2)
                                    wav(:,i) = dG./rt.*(-1).^Nt.*(time_vec(i)-Nt.*pi./protocol.omega(j));  
                                else
                                    wav(:,i) = dG./rt.*(-1).^Nt.*(protocol.smalldel(j)-time_vec(i));
                                end
                            end
                            i_smalldel = i;
                            i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
                        elseif(time_vec(i)<protocol.delta(j))
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                 if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                 if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                             end                        

                        else
                            wav(:,i) = 0;
                        end
                    end
              
            else % waveforms with phase
                rt = rt_vec(j);                
                dG =protocol.G(j).*protocol.grad_dirs(j,:);
                if(protocol.phase(j)>= 0)
                    phdelay = protocol.phase(j)./protocol.omega(j); % time delay due to phase shift
                else
                    phdelay = ((pi()-abs(protocol.phase(j)))./protocol.omega(j));
                    dG = -dG;
                end
                Nt = floor((protocol.smalldel(j)-phdelay).*protocol.omega(j)./pi+0.00000000001);
                    for i=2:length(time_vec)  
                        if time_vec(i)<phdelay
                            if phdelay > 2* rt % duration of the phase shift lrger than 2*rt
                                if(time_vec(i)-rt<-1E-10)
                                     wav(:,i) = dG./rt.*time_vec(i);
                                elseif(time_vec(i)-(phdelay-rt)<-1E-10)
                                      wav(:,i) = dG;
                                else
                                      wav(:,i) = dG./rt.*(phdelay-time_vec(i));
                                end
                            else
                                if(time_vec(i)< phdelay/2)
                                    wav(:,i) = dG./rt.*time_vec(i);                               
                                else
                                    wav(:,i) = dG./rt.*(phdelay-time_vec(i));
                                end
                            end
                            
                        elseif(time_vec(i)<Nt.*pi/protocol.omega(j)+ phdelay)
                            it = floor(protocol.omega(j)*(time_vec(i)-phdelay)./pi()+0.00000001);
                            if( time_vec(i)<it *pi/protocol.omega(j)+rt+phdelay)
                                wav(:,i) = dG./rt.*(-1).^(it+1).*(time_vec(i)-it.*pi./protocol.omega(j)-phdelay);

                            elseif( time_vec(i)<(it+1)*pi/protocol.omega(j)-rt +phdelay)
                                wav(:,i) = dG*(-1)^(it+1);
                            else                   
                                wav(:,i) = dG./rt.*(-1).^(it+1).*((it+1).*pi./protocol.omega(j)+phdelay-time_vec(i));
                            end
                        elseif(time_vec(i)<protocol.smalldel(j)) 
                            if (protocol.smalldel(j)-Nt.*pi/protocol.omega(j)-phdelay>2*rt) % last oscillation longer than 2 *rt
                                if( time_vec(i)<Nt.*pi/protocol.omega(j)+rt+ phdelay)
                                    wav(:,i) = dG./rt.*(-1).^(Nt+1).*(time_vec(i)-Nt.*pi./protocol.omega(j)-phdelay);

                                elseif( time_vec(i)<protocol.smalldel(j)-rt)
                                    wav(:,i) = dG*(-1)^(Nt+1);
                                else
                                    wav(:,i) = dG./rt.*(-1).^(Nt+1).*(protocol.smalldel(j)-time_vec(i));
                                end
                            else  % last oscillation shorter than 2 *rt
                                if( time_vec(i)<protocol.smalldel(j) -(protocol.smalldel(j)-Nt.*pi/protocol.omega(j)-phdelay)/2)
                                    wav(:,i) = dG./rt.*(-1).^(Nt+1).*(time_vec(i)-Nt.*pi./protocol.omega(j)-phdelay);  
                                else
                                    wav(:,i) = dG./rt.*(-1).^(Nt+1).*(protocol.smalldel(j)-time_vec(i));
                                end
                            end
                            i_smalldel = i;
                            i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
                        elseif(time_vec(i)<protocol.delta(j))
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                 if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                 if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                             end                        

                        else
                            wav(:,i) = 0;
                        end
                    end
                
            end
            wave_form(j,:) = wav(:);
            
        end
    else
        error('the protocol does not contain enough information to generate TWOGSE');
    end 
    else % apodised trapezoid; waveforms described in Ianus et al JMR 2012. The area of the first lobe is half the area of a full lobe;
        % to achieve this, the duration of the waveform is increased to
        % smalldel + rt
        rt_vec = protocol.G./protocol.slew_rate; % rise time  
        for j = 1:M
            rt = rt_vec(j);
            dG =protocol.G(j).*protocol.grad_dirs(j,:);
            if protocol.omega(j).*protocol.smalldel(j)/pi - floor(protocol.omega(j).*protocol.smalldel(j)/pi+0.0000000001) > 1E-4; 
                error('apodised cosine works only for waveforms with integer number of lobebs. To use this option set the appropriate omega.')
            end
            if isfield(protocol,'phase')
                if ~(abs(protocol.phase(j) -pi/2) < 1E-4 || abs(protocol.phase(j) + pi/2) < 1E-4) 
                    error('apodisation works only for cosine waveforms, i.e. phase = pi/2 or -pi/2; for a different phase protocol.apodisedcos must be 0')
                end
            end
             for i=1:length(time_vec)
                if(time_vec(i)-(pi/protocol.omega(j)/2+rt/2)<-1E-10)
                    if(time_vec(i)-rt<-1E-10)
                         wav(:,i) = dG./rt.*time_vec(i);
                    elseif(time_vec(i)-(pi/2/protocol.omega(j)+rt/2-rt)<-1E-10)
                          wav(:,i) = dG;
                    else
                          wav(:,i) = dG./rt.*(pi/2./protocol.omega(j)+rt/2-time_vec(i));
                    end
                elseif(time_vec(i)-(protocol.smalldel(j)-pi/protocol.omega(j)/2+rt/2)<-1E-10)
                    it = floor(protocol.omega(j)*(time_vec(i)-pi/protocol.omega(j)/2-rt/2)./pi+0.00000000001);
                    if( (time_vec(i)-pi/protocol.omega(j)/2-rt/2)-(it *pi/protocol.omega(j)+rt)<-1E-10)
                        wav(:,i) = -dG./rt.*(-1).^it.*((time_vec(i)-pi/protocol.omega(j)/2-rt/2)-it.*pi./protocol.omega(j));
                    elseif((time_vec(i)-pi/protocol.omega(j)/2-rt/2)-((it+1)*pi/protocol.omega(j)-rt)<-1E-10)
                        wav(:,i) = -dG*(-1)^it; 
                    else
                        wav(:,i) = -dG./rt.*(-1).^it.*((it+1).*pi./protocol.omega(j)-(time_vec(i)-pi/protocol.omega(j)/2-rt/2));
                    end
                elseif(time_vec(i)-(protocol.smalldel(j)+rt)<-1E-10)
                     it = floor(protocol.omega(j)*(time_vec(i)-pi/protocol.omega(j)/2-rt/2)./pi+0.00000000001);
                     if( (time_vec(i)-pi/protocol.omega(j)/2-rt/2)-(it *pi/protocol.omega(j)+rt)<-1E-10)
                        wav(:,i) = -dG./rt.*(-1).^it.*((time_vec(i)-pi/protocol.omega(j)/2-rt/2)-it.*pi./protocol.omega(j));
                    elseif(time_vec(i) -protocol.smalldel(j)<-1E-10)
                        wav(:,i) = -dG*(-1)^it; 
                    else
                        wav(:,i) = -dG./rt.*(-1).^it.*(protocol.smalldel(j)+rt-time_vec(i));
                     end
                    i_smalldel = i;
                    i_delta = i;
                elseif(time_vec(i)-protocol.delta(j)+rt<-1E-10)
                    wav(:,i) = 0;
                    i_delta = i;
                elseif i-i_delta <=i_smalldel % operate on integers
                    if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                         if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                            wav(:,i) = -wav(:,i-i_delta+1);
                         else
                             wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                         end
                    else
                         if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                            wav(:,i) = -wav(:,i-i_delta);
                         else
                            wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                         end
                     end                        

                else
                    wav(:,i) = 0;
                end
             end
             wave_form(j,:) = wav(:);
        end
        
    end
elseif strcmp(protocol.pulseseq,'SWOGSE_3D')
    total_time = max(ceil((protocol.smalldel+protocol.delta)/protocol.tau)).*protocol.tau+protocol.tau;
    time_vec = 0:protocol.tau:total_time;
    wav = zeros(3,length(time_vec));
    wave_form=zeros(M,length(wav(:)));
    if ~isfield(protocol,'angle') || protocol.angle == 4
        if(isfield(protocol,'omegax') && isfield(protocol,'omegay') && isfield(protocol,'omegaz'))
            if (~isfield(protocol,'phix') && ~isfield(protocol,'phiz') && ~isfield(protocol,'phiy'))
                for j = 1:M
                dG =[protocol.Gx(j); protocol.Gy(j); protocol.Gz(j)];
                   
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([protocol.omegax(j)*time_vec(i)./pi; protocol.omegay(j)*time_vec(i)./pi();...
                                protocol.omegaz(j)*time_vec(i)./pi()]-1E-10);
                            i_smalldel = i+1;
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                 if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                        else
                            wav(:,i) = 0;
                        end                     
                   end   
                   
                   wave_form(j,:) = wav(:);   
                end
            else
                for j = 1:M
                dG =[protocol.Gx(j); protocol.Gy(j); protocol.Gz(j)];
                   
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([(protocol.omegax(j)*time_vec(i)-protocol.phix(j))./pi; (protocol.omegay(j)*time_vec(i)-protocol.phiy(j))./pi();...
                                (protocol.omegaz(j)*time_vec(i)-protocol.phiz(j))./pi()]-1E-10);
                            i_smalldel = i+1;
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                        else
                            wav(:,i) = 0;
                        end                     
                   end 
                    wave_form(j,:) = wav(:);
                  
                end
            end
        else error('the protocol does not contain enough information to generate the waveform')
        end
    elseif protocol.angle == 1
        if(isfield(protocol,'omegax'))
            if (~isfield(protocol,'phix'))
                for j = 1:M
                dG =[protocol.Gx(j); 0; 0];
                  
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([protocol.omegax(j)*time_vec(i)./pi; 0; 0]-1E-10);
                            i_smalldel = i+1;
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                             if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                        else
                            wav(:,i) = 0;
                        end                     
                   end 
                    wave_form(j,:) = wav(:);
                end
                  
            else
                for j = 1:M
                dG =[protocol.Gx(j); 0; 0];
                  
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([(protocol.omegax(j)*time_vec(i)-protocol.phix(j))./pi; 0; 0]-1E-10);
                            i_smalldel = i+1;
                            i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                  else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                                
                        else
                            wav(:,i) = 0;
                        end                     
                   end 
                   wave_form(j,:) = wav(:); 
                  
                end
            end
        else error('the protocol does not contain enough information to generate the waveform');
        end
    elseif protocol.angle == 2
        if(isfield(protocol,'omegax') && isfield(protocol,'omegay') )
            if (~isfield(protocol,'phix')  && ~isfield(protocol,'phiy'))
                for j = 1:M
                dG =[protocol.Gx(j); protocol.Gy(j); 0];                   
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([protocol.omegax(j)*time_vec(i)./pi; protocol.omegay(j)*time_vec(i)./pi; 0]-1E-10);
                            i_smalldel = i+1;
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                        else
                            wav(:,i) = 0;
                        end                     
                   end   
                    wave_form(j,:) = wav(:);
                end               
            else
                for j = 1:M
                dG =[protocol.Gx(j); protocol.Gy(j); 0];
                   
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([(protocol.omegax(j)*time_vec(i)-protocol.phix(j))./pi; (protocol.omegay(j)*time_vec(i)-protocol.phiy(j))./pi; 0]-1E-10);
                            i_smalldel = i+1;
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                        else
                            wav(:,i) = 0;
                        end                     
                   end   
                    wave_form(j,:) = wav(:);
                end
            end
        else error('the protocol does not contain enough information to generate the waveform');
        end
    elseif protocol.angle == 3
        if(isfield(protocol,'omegax')  && isfield(protocol,'omegaz'))
            if (~isfield(protocol,'phix') && ~isfield(protocol,'phiz') )
                for j = 1:M
                dG =[protocol.Gx(j); 0; protocol.Gz(j)];
                   
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([protocol.omegax(j)*time_vec(i)./pi; 0;...
                                protocol.omegaz(j)*time_vec(i)./pi]-1E-10);
                            i_smalldel = i+1;
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                        else
                            wav(:,i) = 0;
                        end                     
                   end   
                   
                   wave_form(j,:) = wav(:);   
                end
            else
                for j = 1:M
                dG =[protocol.Gx(j); 0; protocol.Gz(j)];
                   
                   for i=2:length(time_vec)
                        if(time_vec(i)-protocol.smalldel(j)<-1E-10)  
                            wav(:,i) = dG.*(-1).^floor([(protocol.omegax(j)*time_vec(i)-protocol.phix(j))./pi; 0;...
                                (protocol.omegaz(j)*time_vec(i)-protocol.phiz(j))./pi]-1E-10);
                            i_smalldel = i+1;
                        elseif (time_vec(i)-protocol.delta(j)<-1E-10) 
                            wav(:,i) = 0;
                            i_delta = i;
                        elseif i-i_delta <=i_smalldel % operate on integers
                            if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta+1);
                                 else
                                     wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                                 end
                            else
                                  if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                                    wav(:,i) = -wav(:,i-i_delta);
                                 else
                                    wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                                 end
                            end
                        else
                            wav(:,i) = 0;
                        end                     
                   end   
                   wave_form(j,:) = wav(:);
                end
            end
        else error('the protocol does not contain enough information to generate the waveform')
        end
    else error('Unknown protocol.angle')
    end

elseif strcmp(protocol.pulseseq,'STEAM')    
    % stimulated echo sequence with imaging gradients along z direction and
    % diffusion gradients along grad_dirs
    smalldel = protocol.smalldel; % diffusion gradient duration
    sdelc = protocol.sdelc; % duration of crusher gradient
    sdels = protocol.sdels; % duration of slice select gradient
    tau = protocol.tau; % time interval for waveform discretisation
    G = protocol.G; % amplitude of the diffusion gradient
    Gc = protocol.Gc; % amplitude of the crusher gradient
    Gs = protocol.Gs; % amplitude of the slice gradient
    tau1 = protocol.tau1; % time interval between diffusion gradient and first crusher gradient
    tau2 = protocol.tau2; % time interval between the second crusher gradient and the 2nd diffusion gradient
    tm = protocol.tm; % mixing time between the 2 90deg rf pulses
    
    total_time = max(ceil((2*(smalldel+2*sdels+sdelc)+tau1+tau2+tm)/tau)).*tau+10*tau;
    M = length(protocol.smalldel);
    time_vec = 0:tau:total_time;   
    wave_form=zeros(M,3*length(time_vec));       

     for j = 1:M
        Nsdelc = round(sdelc(j)./tau);
        Nsdels = round(sdels(j)./tau);
        Ntau1 = round(tau1(j)./tau);
        Ntaum = round(tm(j)./tau);
        Ntau2 = round(tau2(j)./tau);
         wav = zeros(3,length(time_vec));
        dG =G(j).*protocol.grad_dirs(j,:);
        Nsmalldel = round(smalldel(j)./tau); % number of point in smalldel
        wav(:,2:1+Nsmalldel) = repmat(dG',1,Nsmalldel);
        wav(3,2+Nsmalldel+Ntau1:1+Nsmalldel+Ntau1+Nsdelc) = Gc(j);
        wav(3,2+Nsmalldel+Ntau1+Nsdelc:1+Nsmalldel+Ntau1+Nsdelc+Nsdels) = Gs(j);
        wav(3,2+Nsmalldel+Ntau1+Nsdelc+Nsdels+Ntaum:1+Nsmalldel+Ntau1+Nsdelc+2*Nsdels+Ntaum) = Gs(j);
        wav(3,2+Nsmalldel+Ntau1+Nsdelc+2*Nsdels+Ntaum:1+Nsmalldel+Ntau1+2*Nsdelc+2*Nsdels+Ntaum) = -Gc(j);
        wav(:,2+Nsmalldel+Ntau1+2*Nsdelc+2*Nsdels+Ntaum+Ntau2:1+2*Nsmalldel+Ntau1+2*Nsdelc+2*Nsdels+Ntaum+Ntau2) = -repmat(dG',1,Nsmalldel);       
       
        wave_form(j,:) = wav(:)';
     end    
    
elseif strcmp(protocol.pulseseq,'dPFG')
    M = length(protocol.G);
    tau = protocol.tau;
    total_time = max(ceil((protocol.smalldel+protocol.delta+protocol.tm)/protocol.tau)).*protocol.tau+10*protocol.tau;
    time_vec = 0:tau:total_time;   
    wave_form=zeros(M,3*length(time_vec));
    for j = 1:M
    G = protocol.G(j);
    smalldel = protocol.smalldel(j);
    delta = protocol.delta(j);
    tmix = protocol.tm(j);
    theta = protocol.theta(j); 
    phi = protocol.phi(j); % angles in spherical coord describing the orientation of the first pair of gradients in the lab coordinate system
    theta1 = protocol.theta1(j);
    phi1 = protocol.phi1(j);% angle in spherical coord describing the orientation of the second pair of gradients in the lab coordinate system
   

    t1=smalldel; %time during first lobe of G1
    t2=delta-smalldel; %between the end of the first lobe and the begining of the second lobe of G1
    if tmix<smalldel
      t3=tmix; % during just the second lobe of G1
      t4=smalldel-tmix; % overlap of second lobe of G1 and first lobe of G2
      t5=0; % between the end of the second lobe of G1 and begining of the first lobe of G2
      t6=tmix;% just during the first lobe of G2
    else
      t3=smalldel;
      t4=0;
      t5=tmix-smalldel;
      t6=smalldel;
    end
    t7=delta-smalldel;
    t8=smalldel;
    
    tt1=round(t1/tau);
    tt2=round(t2/tau);
    tt3=round(t3/tau);
    tt4=round(t4/tau);
    tt5=round(t5/tau);
    tt6=round(t6/tau);
    tt7=round(t7/tau);
    tt8=round(t8/tau);     

    G1v=G*[ [0 0 0]' ...
        repmat([cos(phi)*sin(theta) sin(phi)*sin(theta) cos(theta)]',1,tt1)  ...
        repmat([0 0 0]',1,tt2)  ...
        repmat(-[cos(phi)*sin(theta) sin(phi)*sin(theta) cos(theta)]',1,tt3)  ...
        repmat(-[cos(phi)*sin(theta) sin(phi)*sin(theta) cos(theta)]',1,tt4)  ...
        repmat([0 0 0]',1,tt5)  ...
        repmat([0 0 0]',1,tt6)  ...
        repmat([0 0 0]',1,tt7)  ...
        repmat([0 0 0]',1,tt8)  ...
        [0 0 0]'];
    
    G2v=G*[ [ 0 0 0]' ...
        repmat([0 0 0]',1,tt1)  ...
        repmat([0 0 0]',1,tt2)  ...
        repmat([0 0 0]',1,tt3)  ...
        repmat([-cos(phi1)*sin(theta1) -sin(phi1)*sin(theta1) -cos(theta1)]',1,tt4)  ...
        repmat([0 0 0]',1,tt5)  ...
        repmat([-cos(phi1)*sin(theta1) -sin(phi1)*sin(theta1) -cos(theta1)]',1,tt6)  ...
        repmat([0 0 0]',1,tt7)  ...
        repmat([cos(phi1)*sin(theta1) sin(phi1)*sin(theta1) cos(theta1)]',1,tt8)  ...
        [0 0 0]'];
    
    wav=G1v+G2v; 
     wave_form(j,1:3*size(wav,2))=wav(:)';
    end
elseif strcmp(protocol.pulseseq,'Helical')
    M = length(protocol.G);
    tau = protocol.tau;
    total_time = max(ceil((protocol.smalldel+protocol.delta)/protocol.tau)).*protocol.tau+10*protocol.tau;
    time_vec = 0:tau:total_time;   
    wave_form=zeros(M,3*length(time_vec));
    for j = 1:M
        G = protocol.G(j);       
        for i=2:length(time_vec)
            if(time_vec(i)-protocol.smalldel(j)<-1E-10)       
                wav(1,i) =G*cos(protocol.omega(j).*time_vec(i));
                wav(2,i) =G*sin(protocol.omega(j).*time_vec(i));
                wav(3,i) = G*time_vec(i).*protocol.slopez(j);
                i_smalldel = i+1;
                i_delta = i_smalldel; % for the case when delta-smalldel = 0; needed  for some tests
            elseif(time_vec(i)-protocol.delta(j)<-1E-10)    
                wav(:,i) = 0;
                i_delta = i;
           elseif i-i_delta <=i_smalldel % operate on integers
               if i_delta == i_smalldel % needed when delta-smalldel = 0; needed  for some tests
                     if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                        wav(:,i) = -wav(:,i-i_delta+1);
                     else
                         wav(:,i) = -wav(:,i_smalldel-(i-i_delta));
                     end
                else
                     if ~isfield(protocol,'mirror')  || protocol.mirror ==0 % repeated
                        wav(:,i) = -wav(:,i-i_delta);
                     else
                        wav(:,i) = -wav(:,i_smalldel-(i-i_delta)+1);
                     end
                end
            else
                wav(:,i) = 0;
            end  
        end
        wave_form(j,:) = wav(:)';
    end
elseif strcmp(protocol.pulseseq,'DODE')    
    if isfield(protocol,'grad_dirs1') && isfield(protocol,'grad_dirs2') 
        M = size(protocol.grad_dirs1,1);
        total_time = max(ceil((protocol.smalldel1+protocol.tm+protocol.smalldel2)/protocol.tau)).*protocol.tau+protocol.tau;
        time_vec = 0:protocol.tau:total_time;
        wav = zeros(3,length(time_vec));
        wave_form = zeros(M,length(wav(:))); 
        np1 = floor(protocol.smalldel1./protocol.N1./protocol.tau);
        np2 = floor(protocol.smalldel2./protocol.N2./protocol.tau);
        for j = 1:M
        dG1 =protocol.G1(j).*protocol.grad_dirs1(j,:);
        dG2 =protocol.G2(j).*protocol.grad_dirs2(j,:);
        
        wf1 = repmat([1 -1],np1(j),1);
        wf1 = repmat(wf1(:)',1,protocol.N1(j)/2);
        wf2 = repmat([1 -1],np2(j),1);
        wf2 = repmat(wf2(:)',1,protocol.N2(j)/2);
        int = zeros(3,floor(protocol.tm(j)./protocol.tau)+1);
        
        wav(:,2:1+(length(wf1)+length(int)+length(wf2))) = [repmat(dG1',1,length(wf1)).*repmat(wf1,3,1) ...
            int repmat(dG2',1,length(wf2)).*repmat(wf2,3,1)];
        
         wave_form(j,:) = wav(:);
        end
      else
        error('the protocol does not contain enough information to generate DODE');
     end     
elseif strcmp(protocol.pulseseq,'DDE')    
    if isfield(protocol,'grad_dirs1') && isfield(protocol,'grad_dirs2') 
        M = size(protocol.grad_dirs1,1);
        total_time = max(ceil((protocol.delta1+protocol.delta2+protocol.tm+protocol.smalldel2)/protocol.tau)).*protocol.tau+10*protocol.tau;
        time_vec = 0:protocol.tau:total_time;
        wav = zeros(3,length(time_vec));
        wave_form = zeros(M,length(wav(:))); 
        np1 = floor(protocol.smalldel1./protocol.tau);
        np2 = floor(protocol.smalldel2./protocol.tau);
        for j = 1:M
        dG1 =protocol.G1(j).*protocol.grad_dirs1(j,:);
        dG2 =protocol.G2(j).*protocol.grad_dirs2(j,:);
        
        wf1 = ones(3,np1(j));
        wf2 = ones(3,np2(j));
        int1 = zeros(3,floor((protocol.delta1(j)-protocol.smalldel1(j))./protocol.tau)+1);
        int2 = zeros(3,floor((protocol.tm(j)-protocol.smalldel1(j))./protocol.tau)+1);
        int3 = zeros(3,floor((protocol.delta2(j)-protocol.smalldel2(j))./protocol.tau)+1);
        
        wav(:,2:1+(2*length(wf1)+length(int1)+2*length(wf2)+length(int2)+length(int3))) = ...
            [repmat(dG1',1,length(wf1)).*wf1 int1 -repmat(dG1',1,length(wf1)).*wf1 ...
            int2 -repmat(dG2',1,length(wf2)).*wf2 int3 repmat(dG2',1,length(wf2)).*wf2];
        
         wave_form(j,:) = wav(:);
        end
      else
        error('the protocol does not contain enough information to generate DDE');
     end       
else
    error('Unknown protocol.pulseseq');
end
